@@ -14,8 +14,12 @@
 
 """Show the last run loaded into a repository."""
 
+import optparse
+
+import testtools
+
 from testrepository.commands import Command
-from testrepository.results import TestResultFilter
+from testrepository.testcommand import TestCommand
 
 
 class last(Command):
@@ -23,15 +27,37 @@ class last(Command):
 
     Failing tests are shown on the console and a summary of the run is printed
     at the end.
+
+    Without --subunit, the process exit code will be non-zero if the test run
+    was not successful. With --subunit, the process exit code is non-zero if
+    the subunit stream could not be generated successfully.
     """
 
+    options = [
+        optparse.Option(
+            "--subunit", action="store_true",
+            default=False, help="Show output as a subunit stream."),
+        ]
+    # Can be assigned to to inject a custom command factory.
+    command_factory = TestCommand
+
     def run(self):
         repo = self.repository_factory.open(self.ui.here)
-        run_id = repo.latest_id()
-        case = repo.get_test_run(run_id).get_test()
+        testcommand = self.command_factory(self.ui, repo)
+        latest_run = repo.get_latest_run()
+        if self.ui.options.subunit:
+            stream = latest_run.get_subunit_stream()
+            self.ui.output_stream(stream)
+            # Exits 0 if we successfully wrote the stream.
+            return 0
+        case = latest_run.get_test()
+        try:
+            previous_run = repo.get_test_run(repo.latest_id() - 1)
+        except KeyError:
+            previous_run = None
         failed = False
-        output_result = self.ui.make_result(lambda: run_id)
-        result = TestResultFilter(output_result, filter_skip=True)
+        result = self.ui.make_result(
+            latest_run.get_id, testcommand, previous_run=previous_run)
         result.startTestRun()
         try:
             case.run(result)
