@@ -14,14 +14,24 @@
 
 """Load data into a repository."""
 
+from functools import partial
 import optparse
 
-import subunit
-from testtools import ConcurrentTestSuite, MultiTestResult
+import subunit.test_results
+from testtools import ConcurrentTestSuite, MultiTestResult, Tagger
 
+from testrepository.arguments.path import ExistingPathArgument
 from testrepository.commands import Command
 from testrepository.repository import RepositoryNotFound
-from testrepository.results import TestResultFilter
+from testrepository.testcommand import TestCommand
+
+
+def _wrap_result(result, thread_number):
+    worker_id = 'worker-%s' % thread_number
+    tags_to_add = set([worker_id])
+    tags_to_remove = set()
+    return subunit.test_results.AutoTimingTestResultDecorator(
+        Tagger(result, tags_to_add, tags_to_remove))
 
 
 class load(Command):
@@ -35,6 +45,7 @@ class load(Command):
 
     input_streams = ['subunit+']
 
+    args = [ExistingPathArgument('streams', min=0, max=None)]
     options = [
         optparse.Option("--partial", action="store_true",
             default=False, help="The stream being loaded was a partial run."),
@@ -42,7 +53,14 @@ class load(Command):
             "--force-init", action="store_true",
             default=False,
             help="Initialise the repository if it does not exist already"),
+        optparse.Option("--subunit", action="store_true",
+            default=False, help="Display results in subunit format."),
+        optparse.Option("--full-results", action="store_true",
+            default=False,
+            help="Show all test results. Currently only works with --subunit."),
         ]
+    # Can be assigned to to inject a custom command factory.
+    command_factory = TestCommand
 
     def run(self):
         path = self.ui.here
@@ -53,28 +71,46 @@ class load(Command):
                 repo = self.repository_factory.initialise(path)
             else:
                 raise
+        testcommand = self.command_factory(self.ui, repo)
         run_id = None
         # Not a full implementation of TestCase, but we only need to iterate
         # back to it. Needs to be a callable - its a head fake for
         # testsuite.add.
-        cases = lambda:self.ui.iter_streams('subunit')
+        # XXX: Be nice if we could declare that the argument, which is a path,
+        # is to be an input stream.
+        if self.ui.arguments.get('streams'):
+            opener = partial(open, mode='rb')
+            cases = lambda:map(opener, self.ui.arguments['streams'])
+        else:
+            cases = lambda:self.ui.iter_streams('subunit')
         def make_tests(suite):
             streams = list(suite)[0]
             for stream in streams():
                 yield subunit.ProtocolTestCase(stream)
-        case = ConcurrentTestSuite(cases, make_tests)
+        case = ConcurrentTestSuite(cases, make_tests, _wrap_result)
+        # One copy of the stream to repository storage
         inserter = repo.get_inserter(partial=self.ui.options.partial)
-        output_result = self.ui.make_result(lambda: run_id)
-        # XXX: We want to *count* skips, but not show them.
-        filtered = TestResultFilter(output_result, filter_skip=False)
-        filtered.startTestRun()
-        inserter.startTestRun()
+        # One copy of the stream to the UI layer after performing global
+        # filters.
         try:
-            case.run(MultiTestResult(inserter, filtered))
+            previous_run = repo.get_latest_run()
+        except KeyError:
+            previous_run = None
+        output_result = self.ui.make_result(
+            lambda: run_id, testcommand, previous_run=previous_run)
+        result = MultiTestResult(inserter, output_result)
+        result.startTestRun()
+        try:
+            case.run(result)
         finally:
+            # Does not call result.stopTestRun because the lambda: run_id above
+            # needs the local variable to be updated before the
+            # filtered.stopTestRun() call is invoked. This could be fixed by
+            # having a capturing result rather than a lambda, but thats more
+            # code.
             run_id = inserter.stopTestRun()
-            filtered.stopTestRun()
-        if not filtered.wasSuccessful():
+            output_result.stopTestRun()
+        if not output_result.wasSuccessful():
             return 1
         else:
             return 0
