@@ -17,12 +17,19 @@
 import doctest
 
 import testtools
-from testtools.matchers import DocTestMatches
+from testtools.matchers import (
+    DocTestMatches,
+    Equals,
+    )
 
 from testrepository.commands import failing
 from testrepository.ui.model import UI
 from testrepository.repository import memory
-from testrepository.tests import ResourcedTestCase, Wildcard
+from testrepository.tests import (
+    ResourcedTestCase,
+    StubTestCommand,
+    Wildcard,
+    )
 
 
 class TestCommand(ResourcedTestCase):
@@ -51,7 +58,7 @@ class TestCommand(ResourcedTestCase):
         # We should have seen test outputs (of the failure) and summary data.
         self.assertEqual([
             ('results', Wildcard),
-            ('values', [('id', 0), ('tests', 1), ('failures', 1)])],
+            ('summary', False, 1, None, None, None, [('id', 0, None), ('failures', 1, None)])],
             ui.outputs)
         suite = ui.outputs[0][1]
         result = testtools.TestResult()
@@ -77,12 +84,28 @@ class TestCommand(ResourcedTestCase):
         Cases('failing').run(inserter)
         Cases('ok').run(inserter)
         inserter.stopTestRun()
-        self.assertEqual(1, cmd.execute())
+        self.assertEqual(0, cmd.execute())
         self.assertEqual(1, len(ui.outputs))
         self.assertEqual('stream', ui.outputs[0][0])
         self.assertThat(ui.outputs[0][1], DocTestMatches("""...test: ...failing
 ...failure: ...failing...""", doctest.ELLIPSIS))
 
+    def test_with_subunit_no_failures_exit_0(self):
+        ui, cmd = self.get_test_ui_and_cmd(options=[('subunit', True)])
+        cmd.repository_factory = memory.RepositoryFactory()
+        repo = cmd.repository_factory.initialise(ui.here)
+        inserter = repo.get_inserter()
+        inserter.startTestRun()
+        class Cases(ResourcedTestCase):
+            def ok(self):
+                pass
+        Cases('ok').run(inserter)
+        inserter.stopTestRun()
+        self.assertEqual(0, cmd.execute())
+        self.assertEqual(1, len(ui.outputs))
+        self.assertEqual('stream', ui.outputs[0][0])
+        self.assertThat(ui.outputs[0][1], Equals(''))
+
     def test_with_list_shows_list_of_tests(self):
         ui, cmd = self.get_test_ui_and_cmd(options=[('list', True)])
         cmd.repository_factory = memory.RepositoryFactory()
@@ -135,3 +158,13 @@ class TestCommand(ResourcedTestCase):
         self.assertEqual(1, cmd.execute())
         self.assertEqual([True], calls)
 
+    def test_grabs_TestCommand_result(self):
+        ui, cmd = self.get_test_ui_and_cmd()
+        cmd.repository_factory = memory.RepositoryFactory()
+        calls = []
+        cmd.repository_factory.initialise(ui.here)
+        cmd.command_factory = StubTestCommand()
+        cmd.execute()
+        self.assertEqual(
+            [('startTestRun',), ('stopTestRun',)],
+            cmd.command_factory.results[0]._events)
